"""StarLine constants."""
DEFAULT_CONNECT_TIMEOUT = 10
DEFAULT_READ_TIMEOUT = 120
DEFAULT_ENCODING = 'utf-8'

GET = 'GET'
POST = 'POST'

BATTERY_LEVEL_MIN = 11.8
BATTERY_LEVEL_MAX = 13.0

GSM_LEVEL_MIN = 1
GSM_LEVEL_MAX = 30

DEVICE_FUNCTION_POSITION = "position"
DEVICE_FUNCTION_STATE = "state"



"""Base StarLine API."""
import logging
import requests
from datetime import datetime
from typing import Optional
from .const import DEFAULT_CONNECT_TIMEOUT, DEFAULT_READ_TIMEOUT, DEFAULT_ENCODING, GET, POST

_LOGGER = logging.getLogger(__name__)


class BaseApi:
    """Base StarLine API class."""

    def __init__(self):
        """Constructor."""
        self._session: requests.Session = requests.Session()
        self._connect_timeout: int = DEFAULT_CONNECT_TIMEOUT
        self._read_timeout: int = DEFAULT_READ_TIMEOUT
        self._encoding: str = DEFAULT_ENCODING

    def set_timeout(self, read_timeout: int, connect_timeout: int = None) -> None:
        """Set connection timeouts."""
        self._read_timeout = read_timeout
        if connect_timeout is not None:
            self._connect_timeout = connect_timeout

    def set_encoding(self, encoding: str) -> None:
        """Set response encoding."""
        self._encoding = encoding

    def _request(self, method: str, url: str, params: dict = None, data: dict = None, json: dict = None, headers: dict = None) -> Optional[requests.Response]:
        """Make request."""

        try:
            response = self._session.request(
                method,
                url,
                params=params,
                data=data,
                json=json,
                headers=headers,
                timeout=(self._connect_timeout, self._read_timeout),
            )
            response.encoding = self._encoding
            response.raise_for_status()

            _LOGGER.debug("StarlineApi {} request: {}".format(method, url))
            _LOGGER.debug("  Payload: {}".format(params))
            _LOGGER.debug("  Data: {}".format(data))
            _LOGGER.debug("  JSON: {}".format(json))
            _LOGGER.debug("  Headers: {}".format(headers))
            _LOGGER.debug("  Response: {}".format(response))

            return response
        except requests.exceptions.RequestException as error:
            _LOGGER.error("Request failed: %s", error)
            return None

    def _get(self, url: str, params: dict = None, headers: dict = None) -> Optional[dict]:
        """Make GET request."""

        response = self._request(GET, url, params=params, headers=headers)
        if response is None:
            return None

        data = response.json()
        _LOGGER.debug("  Data: {}".format(data))
        return data

    def _post(self, url: str, params: dict = None, data: dict = None, json: dict = None, headers: dict = None) -> Optional[dict]:
        """Make POST request."""

        response = self._request(POST, url, params=params, data=data, json=json, headers=headers)
        if response is None:
            return None

        data = response.json()
        _LOGGER.debug("  Data: {}".format(data))
        return data

    def get_user_id(self, slid_token: str) -> (str, float, str):
        """Authenticate user by StarLineID token."""

        url = "https://developer.starline.ru/json/v2/auth.slid"
        data = {"slid_token": slid_token}
        response = self._request(POST, url, json=data)
        if response is None:
            raise Exception("Failed to get SLNet token")

        json = response.json()
        if int(json["code"]) != 200:
            raise Exception(json["codestring"])

        slnet_token = None
        expires_time = datetime.now().timestamp() + (4 * 60 * 60)  # Now + 4h
        for cookie in response.cookies:
            if cookie.name == 'slnet':
                slnet_token = cookie.value
                expires_time = cookie.expires

        if slnet_token is None:
            raise Exception("Failed to get SLNet token")

        _LOGGER.debug("SLNet token: {}".format(slnet_token))
        return slnet_token, expires_time, json["user_id"]



"""Auth StarLine API."""
import hashlib
import logging
from .base_api import BaseApi

_LOGGER = logging.getLogger(__name__)


class StarlineAuth(BaseApi):
    """Auth StarLine API class."""

    def get_app_code(self, app_id: str, app_secret: str) -> str:
        """Get application code for getting application token."""

        url = "https://id.starline.ru/apiV3/application/getCode/"
        payload = {
            "appId": app_id,
            "secret": hashlib.md5(app_secret.encode(self._encoding)).hexdigest(),
        }
        response = self._get(url, params=payload)
        if response is None:
            raise Exception("Failed to get application code")

        if int(response["state"]) == 1:
            app_code = response["desc"]["code"]
            _LOGGER.debug("Application code: {}".format(app_code))
            return app_code

        raise Exception("Invalid response state: {}", response["state"])

    def get_app_token(self, app_id: str, app_secret: str, app_code: str) -> str:
        """Get application token for authentication."""

        url = "https://id.starline.ru/apiV3/application/getToken/"
        payload = {
            "appId": app_id,
            "secret": hashlib.md5((app_secret + app_code).encode(self._encoding)).hexdigest(),
        }
        response = self._get(url, params=payload)
        if response is None:
            raise Exception("Failed to get application token")

        if int(response["state"]) == 1:
            app_token = response["desc"]["token"]
            _LOGGER.debug("Application token: {}".format(app_token))
            return app_token

        raise Exception("Invalid response state: {}", response["state"])

    def get_slid_user_token(self, app_token: str, user_login: str, user_password: str, sms_code: str = None, captcha_sid: str = None, captcha_code: str = None) -> (bool, dict):
        """Authenticate user by login, password and application token."""

        url = "https://id.starline.ru/apiV3/user/login/"
        payload = {"token": app_token}
        data = {
            "login": user_login,
            "pass": hashlib.sha1(user_password.encode(self._encoding)).hexdigest(),
        }
        if sms_code is not None:
            data["smsCode"] = sms_code
        if (captcha_sid is not None) and (captcha_code is not None):
            data["captchaSid"] = captcha_sid
            data["captchaCode"] = captcha_code
        response = self._post(url, params=payload, data=data)
        if response is None:
            raise Exception("Failed to get user token")

        state = int(response["state"])
        if (state == 1) or (state == 2) or (state == 0 and "captchaSid" in response["desc"]) or (state == 0 and "phone" in response["desc"]):
            return state, response["desc"]

        raise Exception("Invalid response state: {}", state)



from __future__ import print_function
from setuptools import setup
import codecs
import os
import re

here = os.path.abspath(os.path.dirname(__file__))


def read(*parts):
    return codecs.open(os.path.join(here, *parts), 'r').read()


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


long_description = read('README.rst')


setup(
    name='starline',

    version=find_version('starline', '__init__.py'),
    description='Unofficial python library for StarLine API',
    long_description=long_description,
    url='https://github.com/Anonym-tsk/starline/',
    license='Apache License 2.0',

    keywords='starline car security',

    author='Nikolay Vasilchuk',
    author_email='anonym.tsk@gmail.com',

    install_requires=[
        'aiohttp>=3.6.1'
    ],

    packages=['starline'],
    include_package_data=True,
    platforms='any',
    python_requires='>=3.5',

    classifiers=[
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ]
)



"""StarLine device."""
from typing import Optional, Dict, Any, List
from .const import (
    BATTERY_LEVEL_MIN,
    BATTERY_LEVEL_MAX,
    GSM_LEVEL_MIN,
    GSM_LEVEL_MAX,
    DEVICE_FUNCTION_POSITION,
    DEVICE_FUNCTION_STATE,
)


class StarlineDevice:
    """StarLine device class."""

    def __init__(self):
        """Constructor."""
        self._device_id: Optional[str] = None
        self._imei: Optional[str] = None
        self._alias: Optional[str] = None
        self._battery: Optional[int] = None
        self._ctemp: Optional[int] = None
        self._etemp: Optional[int] = None
        self._fw_version: Optional[str] = None
        self._gsm_lvl: Optional[int] = None
        self._phone: Optional[str] = None
        self._status: Optional[int] = None
        self._ts_activity: Optional[float] = None
        self._typename: Optional[str] = None
        self._balance: Dict[str, Dict[str, Any]] = {}
        self._car_state: Dict[str, bool] = {}
        self._car_alr_state: Dict[str, bool] = {}
        self._functions: List[str] = []
        self._position: Dict[str, float] = {}

    def update(self, device_data):
        """Update data from server."""
        self._device_id = str(device_data["device_id"])
        self._imei = device_data["imei"]
        self._alias = device_data["alias"]
        self._battery = device_data["battery"]
        self._ctemp = device_data["ctemp"]
        self._etemp = device_data["etemp"]
        self._fw_version = device_data["fw_version"]
        self._gsm_lvl = device_data["gsm_lvl"]
        self._phone = device_data["phone"]
        self._status = device_data["status"]
        self._ts_activity = device_data["ts_activity"]
        self._typename = device_data["typename"]
        self._balance = device_data["balance"]
        self._car_state = device_data["car_state"]
        self._car_alr_state = device_data["car_alr_state"]
        self._functions = device_data["functions"]
        self._position = device_data["position"]

    def update_car_state(self, car_state):
        """Update car state from server."""
        for key in car_state:
            if key in self._car_state:
                self._car_state[key] = car_state[key] in ["1", "true", True]

    @property
    def support_position(self):
        """Is position supported by this device."""
        return DEVICE_FUNCTION_POSITION in self._functions

    @property
    def support_state(self):
        """Is state supported by this device."""
        return DEVICE_FUNCTION_STATE in self._functions

    @property
    def device_id(self):
        """Device ID."""
        return self._device_id

    @property
    def fw_version(self):
        """Firmware version."""
        return self._fw_version

    @property
    def name(self):
        """Device name."""
        return self._alias

    @property
    def typename(self):
        """Device type name."""
        return self._typename

    @property
    def position(self):
        """Car position."""
        return self._position

    @property
    def online(self):
        """Is device online."""
        return int(self._status) == 1

    @property
    def battery_level(self):
        """Car battery level."""
        return self._battery

    @property
    def battery_level_percent(self):
        """Car battery level percent."""
        if self._battery > BATTERY_LEVEL_MAX:
            return 100
        if self._battery < BATTERY_LEVEL_MIN:
            return 0
        return round(
            (self._battery - BATTERY_LEVEL_MIN)
            / (BATTERY_LEVEL_MAX - BATTERY_LEVEL_MIN)
            * 100
        )

    @property
    def balance(self):
        """Device balance."""
        return self._balance["active"]

    @property
    def car_state(self):
        """Car state."""
        return self._car_state

    @property
    def alarm_state(self):
        """Car alarm level."""
        return self._car_alr_state

    @property
    def temp_inner(self):
        """Car inner temperature."""
        return self._ctemp

    @property
    def temp_engine(self):
        """Engine temperarure."""
        return self._etemp

    @property
    def gsm_level(self):
        """GSM signal level."""
        return self._gsm_lvl if self.online else 0

    @property
    def gsm_level_percent(self):
        """GSM signal level percent."""
        if self.gsm_level > GSM_LEVEL_MAX:
            return 100
        if self.gsm_level < GSM_LEVEL_MIN:
            return 0
        return round(
            (self.gsm_level - GSM_LEVEL_MIN) / (GSM_LEVEL_MAX - GSM_LEVEL_MIN) * 100
        )

    @property
    def imei(self):
        """Device IMEI."""
        return self._imei

    @property
    def phone(self):
        """Device phone number."""
        return self._phone
